IF (SELECT TOP (1) 1 FROM SysLic) IS NULL
	RETURN;

-- 1. Restore lookup values before update.

DECLARE @forceUpdateSysLookupsValues AS TABLE (Id UNIQUEIDENTIFIER);

-- Lookup values to exclude

/*
INSERT INTO @forceUpdateSysLookupsValues (Id)
Values ('EC4DDB4C-1707-454C-8BDD-A66B45CA154E'),
		('D3D7BFCE-8E7C-468A-8C64-C2865FC55AD6'),
		('C7855AF6-7571-4BD0-B098-4969CEC5B677'),
		('CC181DA2-440E-4B4B-9AC9-8813AF562AA1'),
		('3294DD3A-BE87-4659-B365-E0AB3D11770C'),
		('D6D1EA00-11A9-43CD-B4C4-2DF4B0E95770'),
		--
		('A95B1E9D-5EFC-48A5-A4C2-8878D20C713B'),
		('D577FD71-E039-4E19-BD11-731CD32C17DE'),
		('8AF6722C-B340-4145-B908-49CC74467A68'),
		('E2A22524-1999-4A1E-BA6B-DA5F6BD31DB7'),
		('40E43CA9-CB79-458D-9FAD-403570F113EA'),
		('525F1FE6-F624-445F-BF6D-1D58BA35177A'),
		('97107DCA-EB91-4FDC-ADF4-581B981085AA'),
		('FB8F2F60-F36B-1410-A28E-00FF4C8E32D4'),
		('DDA88D7D-19D0-4009-A3EE-192069FD7E64'),
		--
		('EE1C85C3-CFCB-DF11-9B2A-001D60E938C6'),
		('2795DD03-BACF-E011-92C3-00155D04C01D'),
		('EFE5D7A2-5F38-E111-851E-00155D04C01D'),
		('EA0F3B0A-BACF-E011-92C3-00155D04C01D'),
		('09E4BDA6-CFCB-DF11-9B2A-001D60E938C6'),
		('E7139487-BAD3-E011-92C3-00155D04C01D'),
		('6A8BA927-67CC-DF11-9B2A-001D60E938C6'),
		('E9D91E45-8D92-4E38-95A0-EF8AA28C9E7A'),
		('0DA6A26B-D7BC-DF11-B00F-001D60E938C6'),
		('2B387201-67CC-DF11-9B2A-001D60E938C6'),
		('D4A2DC80-30CA-DF11-9B2A-001D60E938C6'),
		('6A3FB10C-67CC-DF11-9B2A-001D60E938C6'),
		('3DDDB3CC-53EE-49C4-A71F-E9E257F59E49'),
		('9A7AB41B-67CC-DF11-9B2A-001D60E938C6')
*/

DECLARE @id UNIQUEIDENTIFIER,
		@name NVARCHAR(MAX),
		@tableName SYSNAME,
		@sqlCmd NVARCHAR(MAX)

-- Exclude lookup values that need force update

/*
DELETE FROM SysLookupsValues_Backup
WHERE Id IN (SELECT Id FROM @forceUpdateSysLookupsValues);
*/

-- Update statment cursor
DECLARE restoreLookupCursor 
	CURSOR LOCAL FOR
	SELECT DISTINCT 
		slvb.Id, 
		slvb.Name, 
		slvb.TableName
	FROM SysLookupsValues_Backup slvb,
		(SELECT
			Id, 
			MIN(CreatedOn) AS CreatedOn
		FROM SysLookupsValues_Backup 
		GROUP BY Id) AS firstLogValue
	WHERE slvb.Id = firstLogValue.Id
		AND slvb.CreatedOn = firstLogValue.CreatedOn

OPEN restoreLookupCursor;
FETCH NEXT 
	FROM restoreLookupCursor 
	INTO @id, @name, @tableName
WHILE @@FETCH_STATUS = 0
BEGIN
	SET @sqlCmd = '
		IF OBJECT_ID(''CheckValueUpdate' + @tableName + ''') > 0 
		BEGIN
			DROP TRIGGER CheckValueUpdate' + @tableName + ';
		End'
	EXEC sp_sqlexec @sqlCmd;
	SET @sqlCmd = '
		UPDATE '+ @tableName+ '
			SET Name = N''' + @name + '''
		WHERE Id = ''' + CAST(@id AS VARCHAR(36))+ ''''
	BEGIN TRY
		PRINT(@sqlCmd);
		EXEC sp_sqlexec @sqlCmd;
	END TRY
	BEGIN CATCH 
		PRINT ERROR_MESSAGE() + ' while update lookup table '+ @name 
	END CATCH
	FETCH NEXT FROM restoreLookupCursor 
		INTO @id, @name, @tableName
END
DEALLOCATE restoreLookupCursor

-- 2. Restore syssettings values
DECLARE @forceUpdateSysSettingsCode AS TABLE (Code NVARCHAR(500));
INSERT INTO @forceUpdateSysSettingsCode (Code)
VALUES (N'WelcomeScreenVideoUrl'),
		('ConfigurationVersion'),
		('ApiUrl'),
		('ScriptTemplate'),
		('PeriodActualizeBulkEmailRecipients'),
		('SalesDashboardsWithPrefix'),
		('SalesDashboardPrefix'),
		('CaseOrigin'),
		('CaseStatus'),
		('CasePriority'),
		('SatisfactionLevel'),
		('FieldForcePackageUrl')

IF OBJECT_ID('CheckSysSettingsValueUpdate') > 0
BEGIN
	PRINT('DROP TRIGGER [CheckSysSettingsValueUpdate]');
	DROP TRIGGER [CheckSysSettingsValueUpdate];
END

UPDATE SysSettingsValue
SET
	SysAdminUnitId = ussvl.[SysAdminUnitId],
	[IsDef] = ussvl.[IsDef], 
	[TextValue] = ussvl.[TextValue], 
	[IntegerValue] = ussvl.[IntegerValue], 
	[FloatValue] = ussvl.[FloatValue],
	[BooleanValue] = ussvl.[BooleanValue],
	[DateTimeValue] = ussvl.[DateTimeValue],
	[GuidValue] = ussvl.[GuidValue],
	[BinaryValue] = ussvl.[BinaryValue]
FROM SysSettingsValue ssv
INNER JOIN
	(SELECT
		innerLogValue.*
	FROM UpdatedSysSettingsValuesLog innerLogValue,
		(SELECT SysSettingsId,
			MIN(CreatedOn) AS CreatedOn
			FROM UpdatedSysSettingsValuesLog
			GROUP BY SysSettingsId) AS firstLogValue
	WHERE
		innerLogValue.SysSettingsId = firstLogValue.SysSettingsId
		AND innerLogValue.CreatedOn = firstLogValue.CreatedOn) ussvl
	ON ussvl.Id = ssv.Id
INNER JOIN SysSettings ss ON ss.Id = ssv.SysSettingsId
WHERE
	(ussvl.[SysAdminUnitId] <> ssv.[SysAdminUnitId]
	OR ussvl.[IsDef] <> ssv.[IsDef]
	OR ussvl.[TextValue] <> ssv.[TextValue]
	OR ussvl.[IntegerValue] <> ssv.[IntegerValue]
	OR ussvl.[FloatValue] <> ssv.[FloatValue]
	OR ussvl.[BooleanValue] <> ssv.[BooleanValue]
	OR ussvl.[DateTimeValue] <> ssv.[DateTimeValue]
	OR ussvl.[GuidValue] <> ssv.[GuidValue]
	OR ussvl.[BinaryValue] <> ssv.[BinaryValue])
	AND Code NOT IN (SELECT Code FROM @forceUpdateSysSettingsCode);
