IF (SELECT TOP (1) 1 FROM SysLic) IS NULL
	RETURN;

IF OBJECT_ID('SysLookupsValues_Backup') IS NOT NULL
	DROP TABLE SysLookupsValues_Backup;

CREATE TABLE dbo.SysLookupsValues_Backup (
	Id UNIQUEIDENTIFIER NOT NULL,
	Name NVARCHAR(MAX) NOT NULL,
	TableName NVARCHAR(50),
	CreatedOn DATETIME2(3)
) ON [PRIMARY]

DECLARE @Id UNIQUEIDENTIFIER, @Name NVARCHAR(MAX);
DECLARE @cmdDropTrigger NVARCHAR(MAX), @cmdDropTriggerTemplate NVARCHAR(MAX) = 'IF OBJECT_ID(''CheckValueUpdate{Name}'') IS NOT NULL DROP TRIGGER CheckValueUpdate{Name}';
DECLARE @cmdCreateTrigger NVARCHAR(MAX), @cmdCreateTriggerTemplate NVARCHAR(MAX) =
'CREATE TRIGGER CheckValueUpdate{Name}
	 ON {Name}
	 FOR UPDATE
AS
BEGIN
	SET NOCOUNT ON;
	IF UPDATE(Name)
	BEGIN
		INSERT INTO SysLookupsValues_Backup (Id, Name, TableName, CreatedOn)
		SELECT Id, Name, ''{Name}'', GetDate() FROM DELETED;
	END
END';

DECLARE tableNameCursor CURSOR LOCAL FAST_FORWARD FOR
	SELECT
		Id, Name
	FROM SysSchema
	WHERE UId IN (SELECT [SysEntitySchemaUId] FROM Lookup)
		AND Name NOT LIKE 'Vw%'
		AND EXISTS 
			(SELECT TOP (1)
				1
			FROM INFORMATION_SCHEMA.COLUMNS
			WHERE TABLE_NAME = Name
				AND COLUMN_NAME = 'Name');

OPEN tableNameCursor;
FETCH NEXT FROM tableNameCursor INTO @Id, @Name
WHILE @@FETCH_STATUS = 0
BEGIN
	SET @cmdDropTrigger = REPLACE(@cmdDropTriggerTemplate, '{Name}', @Name);
	EXEC (@cmdDropTrigger);

	SET @cmdCreateTrigger = REPLACE(@cmdCreateTriggerTemplate, '{Name}', @Name);
	BEGIN TRY
		EXEC (@cmdCreateTrigger);
	END TRY
	BEGIN CATCH
		PRINT ERROR_MESSAGE();
		PRINT 'on table '+ @Name;
	END CATCH
	FETCH NEXT FROM tableNameCursor INTO @Id, @Name;
END
DEALLOCATE tableNameCursor;

IF OBJECT_ID('UpdatedSysSettingsValuesLog') IS NOT NULL
	DROP TABLE UpdatedSysSettingsValuesLog;

CREATE TABLE [dbo].UpdatedSysSettingsValuesLog
(
	[Id] [uniqueidentifier] NOT NULL DEFAULT (NEWID()),
	[SysSettingsId] [uniqueidentifier] NULL,
	[SysAdminUnitId] [uniqueidentifier] NULL,
	[IsDef] [bit] NOT NULL DEFAULT ((0)),
	[TextValue] [nvarchar](MAX) NOT NULL DEFAULT (''),
	[IntegerValue] [int] NOT NULL DEFAULT ((0)),
	[FloatValue] [decimal](18, 2) NOT NULL DEFAULT ((0)),
	[BooleanValue] [bit] NOT NULL DEFAULT ((0)),
	[DateTimeValue] [datetime2](3) NULL,
	[GuidValue] [uniqueidentifier] NULL,
	[BinaryValue] [varbinary](MAX) NULL,
	[CreatedOn] [datetime2](3)
);

IF OBJECT_ID('CheckSysSettingsValueUpdate') IS NOT NULL
	DROP TRIGGER [CheckSysSettingsValueUpdate];

DECLARE @cmd NVARCHAR(MAX) =
';CREATE TRIGGER CheckSysSettingsValueUpdate
	ON SysSettingsValue
	FOR UPDATE
AS 
BEGIN
	SET NOCOUNT ON;
	INSERT INTO UpdatedSysSettingsValuesLog ([Id]
		,[SysSettingsId]
		,[SysAdminUnitId]
		,[IsDef]
		,[TextValue]
		,[IntegerValue]
		,[FloatValue]
		,[BooleanValue]
		,[DateTimeValue]
		,[GuidValue]
		,[BinaryValue]
		,[CreatedOn])
	SELECT [Id]
		,[SysSettingsId]
		,[SysAdminUnitId]
		,[IsDef]
		,[TextValue]
		,[IntegerValue]
		,[FloatValue]
		,[BooleanValue]
		,[DateTimeValue]
		,[GuidValue]
		,[BinaryValue]
		,GETDATE() FROM DELETED
END';
EXEC(@cmd);
