IF NOT OBJECT_ID('[dbo].[tsp_CreateClusteredPrimaryKeyIndexes]') IS NULL
BEGIN
	DROP PROCEDURE [dbo].[tsp_CreateClusteredPrimaryKeyIndexes]
END
GO

CREATE PROCEDURE [tsp_CreateClusteredPrimaryKeyIndexes] 
	@fillFactor TINYINT = 80,
	@tablesPerIteration INT = -1,
	@doNotExecute BIT = 0,
	@withUpdateStats BIT = 0
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @startDate DATETIME;
	SET @startDate = GETDATE();

	DECLARE @name NVARCHAR(128);
	DECLARE @fillFactorParameter NVARCHAR(3);
	DECLARE @tablesCounter INT = 0;

	IF @doNotExecute = 1
		PRINT 'DEMO MODE (do not alter tables)';

	PRINT 'Database: ' + DB_NAME();

	DECLARE @tablesPerIterationString NVARCHAR(10);
	IF @tablesPerIteration > -1
		SET @tablesPerIterationString = CONVERT(NVARCHAR(10), @tablesPerIteration);
	ELSE
		SET @tablesPerIterationString= 'ALL';
	PRINT 'Will be processed ' + @tablesPerIterationString + ' user tables';

	IF @fillFactor <= 0 OR @fillFactor > 100
		SET @fillFactorParameter = '80';
	ELSE
		SET @fillFactorParameter = CONVERT(NVARCHAR(3), @fillFactor);
	PRINT 'Will be used index fillfactor = ' + @fillFactorParameter;
	PRINT '';

	PRINT 'These user tables already have primary key clustered index and will be skipped:';
	DECLARE skippedTables CURSOR LOCAL FAST_FORWARD FOR
		SELECT [t].[Name]
		FROM [sys].[indexes] [i]
			INNER JOIN [sys].[tables] [t]
				ON [t].[object_id] = [i].[object_id]
		WHERE [i].[index_id] = 1 AND [i].[is_primary_key] = 1
		ORDER BY [t].[Name];

	OPEN skippedTables;
	FETCH NEXT FROM skippedTables INTO @name;
	IF @@FETCH_STATUS <> 0   
        PRINT SPACE(4) + 'Not found'
	WHILE @@FETCH_STATUS = 0
    BEGIN  
		PRINT SPACE(4) + @name;
		FETCH NEXT FROM skippedTables INTO @name;
	END;
	PRINT '';
	CLOSE skippedTables;
	DEALLOCATE skippedTables;

	PRINT 'These user tables already have NON primary key clustered index and will be skipped:';
	DECLARE skippedTables CURSOR LOCAL FAST_FORWARD FOR
		SELECT [t].[Name]
		FROM [sys].[indexes] [i]
			INNER JOIN [sys].[tables] [t]
				ON [t].[object_id] = [i].[object_id]
		WHERE [i].[index_id] = 1 AND [i].[is_primary_key] = 0
		ORDER BY [t].[Name];

	OPEN skippedTables;
	FETCH NEXT FROM skippedTables INTO @name;
	IF @@FETCH_STATUS <> 0   
        PRINT SPACE(4) + 'Not found'
	WHILE @@FETCH_STATUS = 0
    BEGIN  
		PRINT SPACE(4) + @name;
		FETCH NEXT FROM skippedTables INTO @name;
	END;
	PRINT '';
	CLOSE skippedTables;
	DEALLOCATE skippedTables;

	DECLARE @primaryKeys CURSOR;
	SET @primaryKeys = CURSOR LOCAL STATIC FORWARD_ONLY FOR
		SELECT [t].[Name] [Table],
			[i].[name] [Index]
		FROM [sys].[indexes] [i]
			INNER JOIN [sys].[tables] [t]
				ON [t].[object_id] = [i].[object_id]
		WHERE [i].[index_id] > 1 AND [i].[is_primary_key] = 1
			AND NOT EXISTS (SELECT NULL FROM [sys].[indexes] [a] WHERE [a].[index_id] = 1
				AND [a].[object_id] = [t].[object_id])
		ORDER BY [Table];

	PRINT 'Will be converted';
	DECLARE @primaryKeyConstraints TABLE (
		[Index] NVARCHAR(128),
		[Column] NVARCHAR(128),
		[KeyOrdinal] TINYINT,
		[SortOrder] NVARCHAR(6)
	);
	INSERT INTO @primaryKeyConstraints (
		[Index],
		[Column],
		[KeyOrdinal],
		[SortOrder])
	SELECT [i].[name] [Index],
		'[' + COL_NAME([t].[object_id],[ic].[column_id]) [Column],
		[ic].[key_ordinal],
		CASE WHEN [ic].[is_descending_key] = 0 THEN '] ASC' ELSE '] DESC' END
	FROM [sys].[indexes] [i]
		INNER JOIN [sys].[tables] [t]
			ON [t].[object_id] = [i].[object_id]
		INNER JOIN [sys].[index_columns] [ic]
			ON [ic].[object_id] = [i].[object_id] AND [ic].[index_id] = [i].[index_id]
	WHERE [i].[index_id] > 1 AND [i].[is_primary_key] = 1
		AND NOT EXISTS (SELECT NULL FROM [sys].[indexes] [a] WHERE [a].[index_id] = 1
			AND [a].[object_id] = [t].[object_id]);
	PRINT SPACE(4) + CAST(@@ROWCOUNT AS NVARCHAR(10)) + ' primary key constraints';

	DECLARE @foreingKeyConstraints TABLE (
		[Constraint] NVARCHAR(128),
		[Parent_object] NVARCHAR(128),
		[Parent_column] NVARCHAR(128),
		[Referenced_object] NVARCHAR(128),
		[Referenced_column] NVARCHAR(128),
		[Delete_referential_action] NVARCHAR(20),
		[Update_referential_action] NVARCHAR(20)
	);
	INSERT INTO @foreingKeyConstraints (
		[Constraint],
		[Parent_object],
		[Parent_column],
		[Referenced_object],
		[Referenced_column],
		[Delete_referential_action],
		[Update_referential_action])
	SELECT OBJECT_NAME([fkc].[constraint_object_id]),
		OBJECT_NAME([fkc].[parent_object_id]),
		COL_NAME([fkc].[parent_object_id], [fkc].[parent_column_id]),
		OBJECT_NAME([fkc].[referenced_object_id]),
		COL_NAME([fkc].[referenced_object_id], [fkc].[referenced_column_id]),
		CASE WHEN [fk].[delete_referential_action] = 0 THEN '' ELSE ' ON DELETE ' + REPLACE([fk].[delete_referential_action_desc], '_', ' ') END,
		CASE WHEN [fk].[update_referential_action] = 0 THEN '' ELSE ' ON UPDATE ' + REPLACE([fk].[update_referential_action_desc], '_', ' ') END
	FROM [sys].[foreign_key_columns] [fkc]
		INNER JOIN [sys].[tables] [t]
			ON [t].[object_id] = [fkc].[referenced_object_id]
		INNER JOIN [sys].[indexes] [i]
			ON [i].[object_id] = [t].[object_id]
		INNER JOIN [sys].[foreign_keys] [fk]
			ON [fk].[object_id] = [fkc].[constraint_object_id]
	WHERE [i].[index_id] > 1 AND [i].[is_primary_key] = 1;
	PRINT SPACE(4) + CAST(@@ROWCOUNT AS NVARCHAR(10)) + ' foreign key constraints';
	PRINT '';

	DECLARE @sqlStatement NVARCHAR(MAX);
	DECLARE @sqlForeingKeyConstraints TABLE (
		[RowId] INT IDENTITY,
		[Statement] NVARCHAR(MAX)
	);
	DECLARE @sqlPkExtraProperties TABLE (
		[ExtraProperty] NVARCHAR(MAX)
	);
	DECLARE @sqlFkExtraProperties TABLE (
		[ExtraProperty] NVARCHAR(MAX)
	);
	DECLARE @tableName NVARCHAR(128);
	DECLARE @pkConstrain CURSOR;
	DECLARE @pkConstraintName NVARCHAR(128);
	DECLARE @pkConstraintColumnName NVARCHAR(128);
	DECLARE @pkConstraintColumnOrdinal TINYINT;
	DECLARE @pkConstraintColumnSortOrder NVARCHAR(6);
	DECLARE @fkConstraints CURSOR;
	DECLARE @fkConstraintName NVARCHAR(128);
	DECLARE @fkConstraintParentObject NVARCHAR(128);
	DECLARE @fkConstraintParentColumn NVARCHAR(128);
	DECLARE @fkConstraintReferencedObject NVARCHAR(128);
	DECLARE @fkConstraintReferencedColumn NVARCHAR(128);
	DECLARE @deleteReferentialAction NVARCHAR(20);
	DECLARE	@updateReferentialAction NVARCHAR(20);
	DECLARE @extraProperties CURSOR;

	OPEN @primaryKeys;
	FETCH NEXT FROM @primaryKeys INTO @tableName, @pkConstraintName;
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @tablesCounter = @tablesCounter + 1;
		IF @tablesPerIteration > -1 AND @tablesCounter > @tablesPerIteration
			BREAK;

		PRINT 'Table ' + @tableName;
		DELETE FROM @sqlForeingKeyConstraints;
		DELETE FROM @sqlPkExtraProperties;
		DELETE FROM @sqlFkExtraProperties;

		IF EXISTS (SELECT NULL FROM [sys].[columns]
			WHERE OBJECT_NAME([object_id]) = @tableName
				AND [system_type_id] IN (34, 35, 165, 240, 173))
		BEGIN
			PRINT SPACE(4) + 'Table contains column of one of big data types: IMAGE, TEXT, HIERARCHYID, GEOMETRY, GEOGRAPHY, VARBINARY, BINARY';
			PRINT SPACE(4) + 'Create unique clustered index';
			SET @sqlStatement = 'CREATE UNIQUE CLUSTERED INDEX [ClusteredPrimaryKeyIndex] ON [' + @tableName + '] (';
			SET @pkConstrain = CURSOR FAST_FORWARD FOR
				SELECT [Column],
					[SortOrder]
				FROM @primaryKeyConstraints
				WHERE [Index] = @pkConstraintName
				ORDER BY [KeyOrdinal];

			OPEN @pkConstrain;
			FETCH NEXT FROM @pkConstrain INTO @pkConstraintColumnName,
				@pkConstraintColumnSortOrder;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				SET @sqlStatement = @sqlStatement + @pkConstraintColumnName + @pkConstraintColumnSortOrder;
				FETCH NEXT FROM @pkConstrain INTO @pkConstraintColumnName,
					@pkConstraintColumnSortOrder
				IF @@FETCH_STATUS = 0
					SET @sqlStatement = @sqlStatement + ', ';
			END;
			CLOSE @pkConstrain;
			DEALLOCATE @pkConstrain;
			SET @sqlStatement = @sqlStatement + ') WITH (FILLFACTOR = ' + @fillFactorParameter + ');';
			PRINT SPACE(8) + @sqlStatement;
			IF @doNotExecute = 0
				EXECUTE (@sqlStatement);
			FETCH NEXT FROM @primaryKeys INTO @tableName, @pkConstraintName;
			CONTINUE;
		END;

		BEGIN TRY
			BEGIN TRANSACTION;
			PRINT SPACE(4) + 'Drop referencing foreign key constraints';
			SET @fkConstraints = CURSOR FAST_FORWARD FOR
				SELECT [Constraint],
					[Parent_object],
					[Parent_column],
					[Referenced_object],
					[Referenced_column],
					[Delete_referential_action],
					[Update_referential_action]
				FROM @foreingKeyConstraints
				WHERE [Referenced_object] = @tableName;

			OPEN @fkConstraints;
			FETCH NEXT FROM @fkConstraints INTO @fkConstraintName,
				@fkConstraintParentObject,
				@fkConstraintParentColumn,
				@fkConstraintReferencedObject,
				@fkConstraintReferencedColumn,
				@deleteReferentialAction,
				@updateReferentialAction;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				INSERT INTO @sqlForeingKeyConstraints ([Statement])
					VALUES('ALTER TABLE [' + @fkConstraintParentObject +
					'] WITH NOCHECK ADD CONSTRAINT [' + @fkConstraintName + '] FOREIGN KEY([' +
					@fkConstraintParentColumn + ']) REFERENCES [' + @fkConstraintReferencedObject +
					'] ([' + @fkConstraintReferencedColumn + '])' + @deleteReferentialAction + @updateReferentialAction + ';');
				INSERT INTO @sqlForeingKeyConstraints ([Statement])
					VALUES('ALTER TABLE [' + @fkConstraintParentObject + '] CHECK CONSTRAINT [' +
					@fkConstraintName + '];');

				INSERT INTO @sqlFkExtraProperties ([ExtraProperty])
				SELECT 'EXEC sys.sp_addextendedproperty @name=N''' + [name] +
					''', @value=N''' + CONVERT(NVARCHAR(MAX), [value]) + ''' , @level0type=N''SCHEMA'', @level0name=N''' + SCHEMA_NAME() +
					''', @level1type=N''TABLE'', @level1name=N''' + @fkConstraintParentObject +
					''', @level2type=N''CONSTRAINT'', @level2name=N''' + @fkConstraintName + ''';'
				FROM sys.fn_listextendedproperty(NULL, N'SCHEMA', SCHEMA_NAME(), N'TABLE', @fkConstraintParentObject, N'CONSTRAINT', @fkConstraintName);

				SET @sqlStatement = 'ALTER TABLE [' + @fkConstraintParentObject +
					'] DROP CONSTRAINT [' + @fkConstraintName + '];';
				PRINT SPACE(8) + @sqlStatement;
				IF @doNotExecute = 0
					EXECUTE (@sqlStatement);
				FETCH NEXT FROM @fkConstraints INTO @fkConstraintName,
					@fkConstraintParentObject,
					@fkConstraintParentColumn,
					@fkConstraintReferencedObject,
					@fkConstraintReferencedColumn,
					@deleteReferentialAction,
					@updateReferentialAction;
			END;
			CLOSE @fkConstraints;
			DEALLOCATE @fkConstraints;

			PRINT SPACE(4) + 'Drop primary key constraint';
			INSERT INTO @sqlPkExtraProperties ([ExtraProperty])
				SELECT 'EXEC sys.sp_addextendedproperty @name=N''' + [name] +
					''', @value=N''' + CONVERT(NVARCHAR(MAX), [value]) + ''' , @level0type=N''SCHEMA'', @level0name=N''' + SCHEMA_NAME() +
					''', @level1type=N''TABLE'', @level1name=N''' + @tableName +
					''', @level2type=N''CONSTRAINT'', @level2name=N''' + @pkConstraintName + ''';'
				FROM sys.fn_listextendedproperty(NULL, N'SCHEMA', SCHEMA_NAME(), N'TABLE', @tableName, N'CONSTRAINT', @pkConstraintName);

			SET @sqlStatement = 'ALTER TABLE [' + @tableName +
				'] DROP CONSTRAINT [' + @pkConstraintName + '];';
			PRINT SPACE(8) + @sqlStatement;
			IF @doNotExecute = 0
				EXECUTE (@sqlStatement);
		
			PRINT SPACE(4) + 'Add primary key constraint';
			SET @sqlStatement = 'ALTER TABLE [' + @tableName +
				'] ADD CONSTRAINT [' + @pkConstraintName + '] PRIMARY KEY CLUSTERED (';
			SET @pkConstrain = CURSOR FAST_FORWARD FOR
				SELECT [Column],
					[SortOrder]
				FROM @primaryKeyConstraints
				WHERE [Index] = @pkConstraintName
				ORDER BY [KeyOrdinal];

			OPEN @pkConstrain;
			FETCH NEXT FROM @pkConstrain INTO @pkConstraintColumnName,
				@pkConstraintColumnSortOrder;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				SET @sqlStatement = @sqlStatement + @pkConstraintColumnName + @pkConstraintColumnSortOrder;
				FETCH NEXT FROM @pkConstrain INTO @pkConstraintColumnName,
					@pkConstraintColumnSortOrder
				IF @@FETCH_STATUS = 0
					SET @sqlStatement = @sqlStatement + ', ';
			END;
			CLOSE @pkConstrain;
			DEALLOCATE @pkConstrain;
			SET @sqlStatement = @sqlStatement + ') WITH (FILLFACTOR = ' + @fillFactorParameter + ');';
			PRINT SPACE(8) + @sqlStatement;
			IF @doNotExecute = 0
				EXECUTE (@sqlStatement);

			PRINT SPACE(4) + 'Add primary key constraint extra properties';
			SET @extraProperties = CURSOR FAST_FORWARD FOR
				SELECT [ExtraProperty]
				FROM @sqlPkExtraProperties;

			OPEN @extraProperties;
			FETCH NEXT FROM @extraProperties INTO @sqlStatement;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				PRINT SPACE(8) + @sqlStatement;
				IF @doNotExecute = 0
					EXECUTE (@sqlStatement);
				FETCH NEXT FROM @extraProperties INTO @sqlStatement;
			END;
			CLOSE @extraProperties;
			DEALLOCATE @extraProperties;

			PRINT SPACE(4) + 'Add referencing foreign key constraints';
			SET @fkConstraints = CURSOR FAST_FORWARD FOR
				SELECT [Statement]
				FROM @sqlForeingKeyConstraints
				ORDER BY [RowId];

			OPEN @fkConstraints;
			FETCH NEXT FROM @fkConstraints INTO @sqlStatement;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				PRINT SPACE(8) + @sqlStatement;
				IF @doNotExecute = 0
					EXECUTE (@sqlStatement);
				FETCH NEXT FROM @fkConstraints INTO @sqlStatement;
			END;
			CLOSE @fkConstraints;
			DEALLOCATE @fkConstraints;

			PRINT SPACE(4) + 'Add referencing foreign key constraint extra properties';
			SET @extraProperties = CURSOR FAST_FORWARD FOR
				SELECT [ExtraProperty]
				FROM @sqlFkExtraProperties;

			OPEN @extraProperties;
			FETCH NEXT FROM @extraProperties INTO @sqlStatement;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				PRINT SPACE(8) + @sqlStatement;
				IF @doNotExecute = 0
					EXECUTE (@sqlStatement);
				FETCH NEXT FROM @extraProperties INTO @sqlStatement;
			END;
			CLOSE @extraProperties;
			DEALLOCATE @extraProperties;

			PRINT SPACE(4) + 'Done!';
			PRINT '';
			COMMIT TRANSACTION;
		END TRY
		BEGIN CATCH
			IF @@TRANCOUNT > 0
				ROLLBACK;
			PRINT ERROR_MESSAGE();
		END CATCH
		FETCH NEXT FROM @primaryKeys INTO @tableName, @pkConstraintName;
	END;

	CLOSE @primaryKeys;
	DEALLOCATE @primaryKeys;

	IF @withUpdateStats = 1
		EXEC sp_updatestats;

	PRINT '---------------------------------------------------------------';
	PRINT 'Execution time: ' + CONVERT(nvarchar(30), GETDATE() - @startDate, 114);
	PRINT '';
END;
GO

EXEC [tsp_CreateClusteredPrimaryKeyIndexes] @withUpdateStats=1;
